/* 
    Enumeration classes:
    Generic class Enumeration provides iteration capabilities based on enumeration or any other type with static fields;
    Generic class EnumerationIndexedArray implements enumeration-indexed arrays
    Generic class CartesianSquareIndexedArray implements indexed arrays indexed by a Cartesian Square based on an enumeration
    
    Copyright (C) 2008-2010 by Sergey A Kryukov
    http://www.SAKryukov.org
*/

namespace SA.Universal.Enumerations {
    using Cardinal = System.UInt32;

    /// <summary>
    /// EnumerationItem play the same role in <seealso cref="Enumeration">Enumeration</seealso> class as enum members do in their parent enum types.
    /// EnumerationItem provides more comprehensive information and resolves the situation when different enum members have same integer values.
    /// </summary>
    /// <typeparam name="ENUM">Any type; however, only the set of the public static fields of the type are essential</typeparam>
    public sealed class EnumerationItem<ENUM> {
        
        private EnumerationItem() { }

        internal EnumerationItem(string name, Cardinal index, object value, ENUM enumValue) { this.FName = name; this.FIndex = index; this.FValue = value; this.FEnumValue = enumValue; }

        /// <summary>
        /// Name of the static field representing enumeration item.
        /// </summary>
        public string Name { get { return FName; } }
        
        /// <summary>
        /// Index of a static field corresponding to present instance of EnumerationItem in the order it appears in the declaration
        /// </summary>
        public Cardinal Index { get { return FIndex; } }
        
        /// <summary>
        /// Value of the static field representing enumeration member.
        /// If ENUM is not an enumeration type, the type of a static field corresponding to present instance of EnumerationItem may be of the type other then ENUN;
        /// in this case EnumValue is assigned to default(ENUM).
        /// </summary>
        public ENUM EnumValue { get { return FEnumValue; } }

        /// <summary>
        /// Value of a static field corresponding to present instance of EnumerationItem.
        /// The type of Value is always ENUM if ENUM is enumeration type, otherwise it can be of any type.
        /// </summary>
        public object Value { get { return FValue; } }

        #region implementation
        Cardinal FIndex;
        string FName;
        ENUM FEnumValue;
        object FValue;
        #endregion implementation

    } //struct EnumerationItem

} //namespace SA.Universal.Enumerations